"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.default = default_1;
const resolvers_js_1 = require("unplugin-vue-components/dist/resolvers.js");
const nodes_js_1 = require("../tool/nodes.js");
const vite_plugin_compression_1 = __importDefault(require("vite-plugin-compression"));
const vite_1 = __importDefault(require("unplugin-vue-components/vite"));
const vite_2 = __importDefault(require("unplugin-auto-import/vite"));
const vite_plugin_html_1 = require("vite-plugin-html");
const vite_3 = __importDefault(require("unplugin-vue-markdown/vite"));
const vite_plugin_mock_1 = require("vite-plugin-mock");
const setVue_js_1 = require("../route/setVue.js");
const vite_svg_loader_1 = __importDefault(require("vite-svg-loader"));
const config_js_1 = __importDefault(require("./config.js"));
const plugin_vue_1 = __importDefault(require("@vitejs/plugin-vue"));
const alone_js_1 = __importDefault(require("../tool/alone.js"));
const vite_4 = require("vite");
const setPlugin = function (list) {
    return Object.keys(list).reduce((acc, key) => {
        acc[key] = function (...data) {
            return { plugin_name: key, ['plugin_' + key]: data };
        };
        return acc;
    }, {});
};
const callPlugin = function (list) {
    return Object.values(list).map(method => method());
};
const callPlugins = function (list) {
    return __awaiter(this, void 0, void 0, function* () {
        return yield Promise.all(callPlugin(list));
    });
};
const framePlugin = function (then) {
    return {
        vue(data) {
            return (0, plugin_vue_1.default)(alone_js_1.default.array_merge({ include: [/\.vue$/, /\.md$/] }, data));
        },
        html(data) {
            const tags = alone_js_1.default.get_arr(data, 'inject.tags');
            const config = alone_js_1.default.array_merge(data, { minify: true, inject: { tags: [] } });
            config.inject.tags = (alone_js_1.default.isset(tags) && alone_js_1.default.is_array(tags)) ? then.html_tag.concat(tags) : then.html_tag;
            config.entry = then.option.main;
            return (0, vite_plugin_html_1.createHtmlPlugin)(config);
        },
        auto(data) {
            return (0, vite_2.default)(alone_js_1.default.array_merge({
                dts: then.env.dts.auto || false,
                imports: ['vue', {
                        'naive-ui': [
                            'useDialog',
                            'useMessage',
                            'useNotification',
                            'useLoadingBar'
                        ]
                    }]
            }, data || {}));
        },
        component(data) {
            return (0, vite_1.default)(alone_js_1.default.array_merge({
                dts: then.env.dts.component || false,
                extensions: ['vue', 'md'],
                include: [/\.vue$/, /\.vue\?vue/, /\.md$/],
                resolvers: [(0, resolvers_js_1.NaiveUiResolver)()],
            }, data || {}));
        },
        svg(data) {
            return (0, vite_svg_loader_1.default)(alone_js_1.default.array_merge({ defaultImport: 'url' }, data));
        },
        md(data) {
            then.plugins.markdown = true;
            const config = { headEnabled: true, markdownItOptions: { html: true, linkify: true, typographer: true } };
            const include = [`${then.node.get_path('.')}/*.md`, `${then.node.get_path('.')}/**/*.md`];
            const includes = alone_js_1.default.get_arr(data, 'include');
            config.include = (alone_js_1.default.isset(includes) && alone_js_1.default.is_array(includes)) ? include.concat(includes) : include;
            return (0, vite_3.default)(config);
        },
        mock(data) {
            if (!then.env.mock)
                return {};
            return (0, vite_plugin_mock_1.viteMockServe)(alone_js_1.default.array_merge({
                mockPath: then.env.mock, //存放 Mock 文件的路径。默认情况下，插件会在此路径下查找 Mock 文件
                watchFiles: true, //指示是否监视 Mock 文件的变化。如果设置为 true，当 Mock 文件发生变化时，Mock 服务器会自动更新。
                enable: true, //指示是否启用 Mock 功能。设置为 false 可以临时禁用 Mock。
                logger: false, //指示是否启用插件的日志记录。设置为 true 会在终端输出 Mock 请求的日志信息。
                cors: true //指示是否启用 CORS（跨域资源共享）。设置为 true 会在响应中添加 CORS 头部，以允许跨域请求。
            }, data || {}));
        },
        compress(data) {
            return (0, vite_plugin_compression_1.default)(alone_js_1.default.array_merge({
                verbose: true, // 默认即可
                disable: true, //开启压缩(不禁用)，默认即可
                deleteOriginFile: false, //删除源文件
                threshold: 10240, //压缩前最小文件大小
                algorithm: 'gzip', //压缩算法
                ext: '.gz', //文件类型
            }, data));
        }
    };
};
const alonePlugin = function (then) {
    return {
        //设置配置
        alone_config() {
            return {
                enforce: 'pre',
                name: 'alone-config',
                config(config) {
                    config.base = config.base || "./";
                    config.resolve = config.resolve || {};
                    config.envPrefix = config.envPrefix || then.env.prefix;
                    config.root = config.root || then.app_path;
                    config.publicDir = config.publicDir || then.publicDir();
                    config.envDir = config.envDir || then.app.get_path('config');
                    config.define = alone_js_1.default.array_merge(then.define(), config.define);
                    config.server = alone_js_1.default.array_merge(then.server(), config.server);
                    const preview = then.preview();
                    config.preview = config.preview || {};
                    config.preview.host = config.preview.host || preview.host;
                    config.preview.port = config.preview.port || preview.port;
                    config.build = alone_js_1.default.array_merge(then.plugin_build(), config.build);
                    config.build.outDir = then.out_dir;
                    config.resolve.alias = alone_js_1.default.array_merge(then.alias(), config.resolve.alias);
                },
                buildStart() {
                    (0, setVue_js_1.delView)();
                    if (then.update) {
                        then.is_del_vite = true;
                        then.node.deletes_file('node_modules/.vite');
                    }
                }
            };
        },
        //加载main
        alone_main() {
            return {
                enforce: 'pre',
                name: 'alone-main',
                transform(code, file) {
                    if (then.trim_rep(file) != then.trim_rep(then.app.get_path(then.option.main)))
                        return null;
                    return {
                        code: `
            import MainLoadBeforeAfterCode from "alone-frame/frame/main";
            MainLoadBeforeAfterCode.status = {};
            await MainLoadBeforeAfterCode.load();
            \n${code}\n
            MainLoadBeforeAfterCode.mount();
            `, map: null
                    };
                }
            };
        },
        //显示
        alone_show() {
            return {
                enforce: 'post',
                name: 'alone-show',
                buildStart() {
                    console.log('path', `${(0, nodes_js_1.is_mac)() ? "file://" : ""}${then.app.get_path('.')}`);
                },
                closeBundle() {
                    console.log('complete', `${(0, nodes_js_1.is_mac)() ? "file://" : ""}` + then.out_dir);
                }
            };
        },
        //路由
        alone_route() {
            return {
                enforce: 'pre',
                name: 'alone-route',
                buildStart() {
                    then.vue_route();
                }
            };
        },
        alone_build() {
            return {
                enforce: 'post',
                apply: 'build',
                name: 'alone-build',
                buildStart() {
                    if (!then.is_del_vite) {
                        then.node.deletes_file('node_modules/.vite');
                    }
                    (0, nodes_js_1.nodes)(then.out_dir).del_dir_file('.', then.env.exclude);
                },
                closeBundle() {
                    (0, nodes_js_1.nodes)(then.out_dir).delete_file('layui/lay.js');
                    if (then.env.zip === true) {
                        (0, nodes_js_1.zip_file)(then.out_dir, alone_js_1.default.rtrim(then.out_dir, '/') + ".zip", function (size, unit, file) {
                            console.log('zip file', file);
                            console.log('zip size', `${size}${unit}`);
                        });
                    }
                }
            };
        }
    };
};
const expandPlugin = function (then) {
    return {
        alone_file_reload(vite_status = false, config_status = false, update = true) {
            return {
                name: 'alone-file-reload',
                handleHotUpdate({ file, server }) {
                    if (vite_status && config_status && update) {
                        const files = then.trim_rep(file);
                        const include = [then.trim_rep(then.app.get_path(then.option.conf)), then.trim_rep(then.app.get_path(then.option.env))];
                        const includes = [then.trim_rep(then.app.get_path(then.option.vite))];
                        if (vite_status && includes.includes(files)) {
                            update = false;
                            server.restart();
                        }
                        else if (config_status && include.includes(files)) {
                            update = false;
                            server.ws.send({ type: 'full-reload' });
                        }
                        if (!update) {
                            console.log(`\x1b[90m${new Date().toLocaleTimeString()}\x1b[0m [alone] \x1b[32mpage reload\x1b[0m ${files.split('/').pop()}`);
                        }
                    }
                },
            };
        }
    };
};
const viteConfig = function (then, pluginList, pluginSet, viteRes) {
    return __awaiter(this, void 0, void 0, function* () {
        let list_plugin = {}, load_plugin = {}, plugins = [];
        if (alone_js_1.default.isset(viteRes)) {
            if (alone_js_1.default.is_array(viteRes.plugins) && alone_js_1.default.isset(viteRes.plugins)) {
                for (const [key, val] of Object.entries(viteRes.plugins)) {
                    const name = val.plugin_name;
                    if (name && pluginList.hasOwnProperty(name)) {
                        list_plugin[name] = val['plugin_' + name];
                        if (list_plugin[name] !== false) {
                            load_plugin[name] = yield (pluginList[name])(...list_plugin[name]);
                        }
                    }
                    else {
                        load_plugin[key] = val;
                    }
                }
                if (alone_js_1.default.isset(list_plugin)) {
                    Object.keys(pluginList).forEach((key) => {
                        if (!list_plugin.hasOwnProperty(key)) {
                            plugins.push(pluginList[key]());
                        }
                    });
                }
                else {
                    plugins = plugins.concat(yield callPlugins(pluginList));
                }
                Object.keys(load_plugin).forEach((key) => plugins.push(load_plugin[key]));
            }
        }
        viteRes.plugins = (alone_js_1.default.isset(plugins) ? plugins : yield callPlugins(pluginList));
        return viteRes;
    });
};
function default_1(pathname = 'src', callback = null) {
    return (0, vite_4.defineConfig)(function (option) {
        return __awaiter(this, void 0, void 0, function* () {
            const then = yield (0, config_js_1.default)(pathname || 'src');
            const pluginList = Object.assign(framePlugin(then), alonePlugin(then), expandPlugin(then));
            const pluginSet = setPlugin(pluginList);
            const call = alone_js_1.default.is_callback(callback) ? callback : then.conf_vite;
            const viteRes = alone_js_1.default.is_callback(call) ? (yield call(option, then, pluginSet)) : {};
            return yield viteConfig(then, pluginList, pluginSet, viteRes);
        });
    });
}
