"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.npm_push_index = exports.nodes = exports.zip_file = exports.file_url = exports.confirm = exports.edit_ver = exports.system_file = exports.is_linux = exports.is_mac = exports.is_win = exports.dir_name = void 0;
const child_process_1 = require("child_process");
const readline_1 = require("readline");
const url_1 = require("url");
const archiver_1 = __importDefault(require("archiver"));
const path_1 = __importDefault(require("path"));
const fs_1 = __importDefault(require("fs"));
const os_1 = __importDefault(require("os"));
//删除路径多少层
const dir_name = function (file, levels = 1) {
    for (let i = 0; i < levels; i++) {
        file = path_1.default.dirname(file);
    }
    return file;
};
exports.dir_name = dir_name;
//判断是否windows
const is_win = function () {
    return os_1.default.platform() === 'win32';
};
exports.is_win = is_win;
//判断是否mac
const is_mac = function () {
    return os_1.default.platform() === 'darwin';
};
exports.is_mac = is_mac;
//判断是否linux
const is_linux = function () {
    return os_1.default.platform() === 'linux';
};
exports.is_linux = is_linux;
//node本地运行await import要使用此方法,解决windows问题
const system_file = function (file) {
    return (0, exports.is_win)() ? (file.includes("file://") ? file : ("file://" + file.replace(/\\/g, '/'))) : file;
};
exports.system_file = system_file;
//增加版本号
const edit_ver = function (ver, add = '0.0.1') {
    const version = ver.split('.').map(Number);
    const increase = add.split('.').map(Number);
    while (version.length < increase.length)
        version.push(0);
    while (increase.length < version.length)
        increase.push(0);
    const parts = version.map((num, index) => num + increase[index]);
    for (let i = parts.length - 1; i > 0; i--) {
        if (parts[i] >= 10) {
            parts[i] -= 10;
            parts[i - 1] += 1;
        }
    }
    return parts.join('.');
};
exports.edit_ver = edit_ver;
//确认输入操作
const confirm = function (title, callback) {
    const rl = (0, readline_1.createInterface)({ input: process.stdin, output: process.stdout });
    rl.question(title, (answer) => {
        callback(answer.toLowerCase());
        rl.close();
    });
};
exports.confirm = confirm;
//获取当前模块
const file_url = function (url = '', levels = 1) {
    return (0, exports.dir_name)((0, url_1.fileURLToPath)((url || import.meta.url)), levels);
};
exports.file_url = file_url;
//打包zip
const zip_file = function (dir, file = '', close = null) {
    file = (file || (dir.replace(/\\/g, '/').replace(/\/+$/, '') + ".zip"));
    const output = fs_1.default.createWriteStream(file);
    const archive = (0, archiver_1.default)('zip', { zlib: { level: 9 } });
    output.on('close', () => {
        const sizeInBytes = archive.pointer();
        let size;
        let unit;
        if (sizeInBytes < 1024) {
            size = sizeInBytes;
            unit = 'Bytes';
        }
        else if (sizeInBytes < 1024 * 1024 * 10) {
            size = (sizeInBytes / 1024).toFixed(2);
            unit = 'KB';
        }
        else {
            size = (sizeInBytes / (1024 * 1024)).toFixed(2);
            unit = 'MB';
        }
        ((close && typeof close == 'function') && close(size, unit, file));
    });
    archive.on('error', (err) => {
        throw err;
    });
    archive.pipe(output);
    archive.directory(dir, false);
    archive.finalize();
};
exports.zip_file = zip_file;
//操作方法
const nodes = function (dirname) {
    const self = { dirname: dirname || process.cwd(), fs: fs_1.default, path: path_1.default, os: os_1.default, exec: child_process_1.execSync, readline: readline_1.createInterface };
    //打包zip
    self.zip_file = function (dir, type = true, file, close = null) {
        (0, exports.zip_file)(self.get_path(dir, type), file, close);
    };
    //获取文件名
    self.get_path = function (file, type = true) {
        return type === true ? path_1.default.resolve(self.dirname, file) : file;
    };
    //判断文件或者目录是否存在
    self.is_exist = function (file, type = true) {
        return fs_1.default.existsSync(self.get_path(file, type));
    };
    //删除路径多少层
    self.dir_name = function (file, levels = 1) {
        return (0, exports.dir_name)(file, levels);
    };
    //创建目录
    self.create_dir = function (dir, type = true) {
        if (!self.is_exist(dir, type)) {
            fs_1.default.mkdirSync(self.get_path(dir, type), { recursive: true });
        }
    };
    //删除目录
    self.delete_dir = function (dir, type = true) {
        if (self.is_exist(dir, type)) {
            fs_1.default.rmdirSync(self.get_path(dir, type));
        }
    };
    //删除文件
    self.delete_file = function (file, type = true) {
        if (self.is_exist(file, type)) {
            fs_1.default.unlinkSync(self.get_path(file, type));
        }
    };
    //删除文件
    self.deletes_file = function (file, type = true) {
        if (self.is_exist(file, type)) {
            fs_1.default.rmSync(self.get_path(file, type), { recursive: true, force: true });
        }
    };
    //目录内数量
    self.dir_count = function (dir, type = true) {
        if (self.is_exist(self.get_path(dir, type))) {
            const entries = fs_1.default.readdirSync(self.get_path(dir, type), { withFileTypes: true });
            return entries.length;
        }
        return 0;
    };
    //获取目录和文件的信息
    self.dir_file_info = function (dir, type = true) {
        return fs_1.default.statSync(self.get_path(dir, type));
    };
    //复制指定文件
    self.cp_file = function (old_file, new_file, old_type = true, new_type = true) {
        self.create_dir(self.dir_name(new_file), new_type);
        if (self.is_exist(old_file, old_type)) {
            fs_1.default.copyFileSync(self.get_path(old_file, old_type), self.get_path(new_file, new_type));
        }
    };
    //复制目录,支持排除文件或者目录
    self.cp_dir_file = function (source, target, exclude = [], source_type = true, target_type = true, old_dir = '') {
        source = self.get_path(source, source_type);
        if (self.is_exist(source, source_type)) {
            self.create_dir(target, target_type);
            target = self.get_path(target, target_type);
            const list = fs_1.default.readdirSync(source, { withFileTypes: true });
            for (const val of list) {
                const dir_file_name = (val.parentPath + "/" + val.name).replace(old_dir ? old_dir : source, '').replace(/^\/|\/$/g, '');
                if (exclude.includes(dir_file_name)) {
                    continue;
                }
                const sour = path_1.default.join(source, val.name);
                const dest = path_1.default.join(target, val.name);
                if (val.isDirectory()) {
                    self.cp_dir_file(sour, dest, exclude, false, false, old_dir ? old_dir : source);
                }
                else {
                    fs_1.default.copyFileSync(sour, dest);
                }
            }
        }
    };
    self.copy_list = function (list, type = false, paths = null) {
        Object.keys(list).forEach((file) => {
            fs_1.default.stat(file, (err, stats) => {
                if (!err) {
                    const files = (list[file] === true ? file : list[file]);
                    if (stats.isDirectory()) {
                        self.cp_dir_file(file, ((paths && typeof paths == 'string') ? path_1.default.resolve(paths, files) : files), [], type, paths === true);
                    }
                    else {
                        self.cp_file(file, ((paths && typeof paths == 'string') ? path_1.default.resolve(paths, files) : files), type, paths === true);
                    }
                }
            });
        });
    };
    //复制目录的文件
    self.copy_dir_file = function (dir, file) {
        if (dir && typeof dir == 'object') {
            Object.keys(dir).forEach((key) => {
                self.cp_dir_file(key, (dir[key] === true ? key : dir[key]), [], false, false);
            });
        }
        if (file && typeof file == 'object') {
            Object.keys(file).forEach((key) => {
                self.cp_file(key, (file[key] === true ? key : file[key]), false, false);
            });
        }
    };
    //删除目录内文件,支持排除文件或者目录
    self.del_dir_file = function (dir, exclude = [], type = true, old_dir = '') {
        dir = self.get_path(dir, type);
        if (self.is_exist(dir, false)) {
            const files = fs_1.default.readdirSync(dir);
            for (const file of files) {
                const filePath = path_1.default.join(dir, file);
                const dir_file_name = filePath.replace(old_dir ? old_dir : dir, '').replace(/^\/|\/$/g, '');
                if (exclude && exclude.includes(dir_file_name)) {
                    continue;
                }
                if (fs_1.default.statSync(filePath).isDirectory()) {
                    self.del_dir_file(filePath, exclude, false, old_dir ? old_dir : dir);
                    if (self.dir_count(filePath, false) <= 0) {
                        fs_1.default.rmdirSync(filePath);
                    }
                }
                else {
                    fs_1.default.unlinkSync(filePath);
                }
            }
        }
    };
    //保存文件
    self.save_file = function (file, data, type = true) {
        const files = self.get_path(file, type);
        self.create_dir(self.dir_name(files), false);
        fs_1.default.writeFileSync(files, data);
        return files;
    };
    //获取文件
    self.get_file = function (file, type = true) {
        const files = self.get_path(file, type);
        if (self.is_exist(files, false)) {
            return fs_1.default.readFileSync(files, 'utf-8');
        }
        return null;
    };
    //保存json
    self.save_json = function (file, json, type = true) {
        try {
            return self.save_file(file, JSON.stringify(json, null, 2), type);
        }
        catch (e) {
            return self.get_path(file, type);
        }
    };
    //获取json
    self.get_json = function (file, type = true) {
        const data = self.get_file(file, type);
        try {
            if (data) {
                return JSON.parse(data);
            }
            return {};
        }
        catch (e) {
            return {};
        }
    };
    self.get_pack = function () {
        return self.get_json('package.json');
    };
    self.save_pack = function (data) {
        return self.save_json('package.json', data);
    };
    return self;
};
exports.nodes = nodes;
//上传和安装
const npm_push_index = ({ src, ver, app_name, version, push, create, pathname, dirname, npm, copy_list } = {}) => {
    const self = {
        src: src || 'src',
        pathname: pathname || (0, exports.file_url)(),
        node: (0, exports.nodes)(pathname || (0, exports.file_url)()),
        dirname: (dirname || process.cwd()),
        del_src() {
            this.node_src = (0, exports.nodes)(this.node.get_path((this.src)));
            this.node.del_dir_file(this.src);
            this.node.delete_dir(this.src);
            return this;
        },
        del_name() {
            this.node_src = (0, exports.nodes)(this.node.get_path((this.src)));
            this.node_src.del_dir_file(this.pack.name);
            this.node_src.delete_dir(this.pack.name);
            return this;
        },
        copy_src({ path, file = [], dir = [] } = {}) {
            this.node_app = (0, exports.nodes)(path);
            if (!this.node_app.is_exist('package.json')) {
                console.log("copy package not exist");
                return;
            }
            this.node_src = (0, exports.nodes)(this.node.get_path((this.src)));
            this.node_name = (0, exports.nodes)(this.node_src.get_path((this.pack.name)));
            const dir_file = (val) => {
                return this.node_name.get_path((val || '.').replace(/^\/+|\/+$/g, ''));
            };
            if (dir) {
                const dir_list = Array.isArray(dir) ? dir : Object.values(dir);
                dir_list.forEach((val) => {
                    this.node_app.cp_dir_file(val, dir_file(val), [], true, false);
                });
            }
            if (file) {
                const file_list = Array.isArray(file) ? file : Object.values(file);
                file_list.forEach((val) => {
                    this.node_app.cp_file(val, dir_file(val), true, false);
                });
            }
            return this;
        },
        push() {
            this.type = 'push';
            if (!this.node.is_exist('package.json')) {
                this.node = (0, exports.nodes)(this.dirname);
                if (!this.node.is_exist('package.json')) {
                    this.node = null;
                }
            }
            if (!this.node) {
                console.log("package not exist");
                return this;
            }
            //获取package
            this.pack = this.node.get_pack() || {};
            this.pack.version = (version || (0, exports.edit_ver)(ver || this.pack.version || "1.0.0", '0.0.1'));
            this.pack.type = this.pack.type || "module";
            this.pack.license = this.pack.license || "license";
            this.pack.author = this.pack.author || "You";
            this.pack.scripts = this.pack.scripts || {};
            this.pack.scripts.push = this.pack.scripts.push || "node push.js";
            //是否create和显不的包名
            if (this.pack.name.startsWith("create")) {
                this.pack_create = true;
                this.pack_name = this.pack.name.split('-').slice(1).join('-');
                this.pack.files = [...new Set([...(this.pack.files || []), ...[this.src, "index.js"]])];
                this.pack.bin = this.pack.bin || {};
                this.pack.bin.cva = this.pack.bin.cva || "index.js";
                this.pack.bin[this.pack.name] = this.pack.bin[this.pack.name] || "index.js";
                this.pack.engines = this.pack.engines || { "node": "^18.0.0 || >=20.0.0" };
            }
            else {
                this.pack_create = false;
                this.pack_name = this.pack.name;
            }
            this.pack.dependencies = this.pack.dependencies || { "@types/node": "^22.8.4" };
            (typeof push == 'function' && push(this, 1));
            console.log("\nnpm whoami");
            console.log("\nnpm publish");
            console.log("\nnpm config set //registry.npmjs.org/:_authToken <token>");
            console.log("\nhttps://www.npmjs.com/package/" + this.pack.name + "?activeTab=code\n");
            (0, exports.confirm)('publish ? (y/n): ', (val) => {
                if (val == 'y') {
                    this.node.save_pack(this.pack);
                    this.node.exec('npm publish');
                    if (this.pack_create) {
                        console.log("\npnpm create " + this.pack_name + "@" + this.pack.version);
                        console.log("\nnpm create " + this.pack_name + "@latest");
                    }
                    else {
                        console.log("\npnpm install " + this.pack.name + "@" + this.pack.version);
                        console.log("\nnpm install " + this.pack.name + "@latest");
                    }
                    console.log("\nhttps://www.npmjs.com/package/" + this.pack.name + "?activeTab=code\n");
                    (typeof push == 'function' && push(this, 2));
                }
            });
            return this;
        },
        //获取npm安装目录
        get_npm_dir() {
            this.type = 'create';
            if (!this.node.is_exist('package.json')) {
                this.node = (0, exports.nodes)((0, exports.dir_name)(process.argv[1]));
                if (!this.node.is_exist('package.json')) {
                    const key = {
                        INIT_CWD: false,
                        PWD: false,
                        npm_config_local_prefix: false,
                        npm_package_json: true
                    };
                    for (const k in key) {
                        if (process.env[k]) {
                            this.node = (0, exports.nodes)(((key[k] === true) ? (0, exports.dir_name)(process.env[k]) : process.env[k]));
                            if (this.node.is_exist('package.json')) {
                                break;
                            }
                            else {
                                this.node = null;
                            }
                        }
                    }
                }
            }
            //找不到安装目录
            if (!this.node) {
                console.log("npm directory not exist", process);
                return this;
            }
            //用户目录
            this.user = (0, exports.nodes)(this.dirname);
            //用户输入的项目名
            this.app_name = process.argv[2] || app_name;
            if (!this.app_name) {
                this.app_arr = this.node.get_path('.').replace(/\\/g, '/').split('/').pop().split('-');
                this.app_name = this.app_arr.length > 1 ? this.app_arr.slice(1).join('-') : this.app_arr.join('-');
            }
            this.app_name = this.app_name.replace(/\\/g, '/').replace(/^\/+|\/+$/g, '');
            //删除用户目录
            this.user.del_dir_file(this.app_name);
            (typeof create == 'function' && create(this, 1));
            return this;
        },
        get_command() {
            //获取用户使用的命令类型
            this.command = { name: "npm", list: ['pnpm', 'yarn', 'bun', 'npm'] };
            this.npm_dir = this.node.get_path('.').replace(/\\/g, '/');
            for (const val of this.command.list) {
                if (this.npm_dir.includes(val + '-') ||
                    this.npm_dir.includes('.' + val) ||
                    this.npm_dir.includes('/' + val) ||
                    this.npm_dir.includes(val + '/')) {
                    this.command.name = val;
                    break;
                }
            }
            console.log("\ncd " + this.app_name + "\n");
            console.log(this.command.name + " install\n");
            console.log(this.command.name + " run dev\n");
            return this;
        },
        create_src() {
            //src目录
            this.node_src = (0, exports.nodes)(this.node.get_path((this.src)));
            //要复制的目录
            this.app_dir = process.argv[3] || '';
            //判断目录是否存在
            if (this.app_dir) {
                if (!this.node_src.is_exist(this.app_dir)) {
                    this.app_dir = null;
                }
            }
            //要复制的目录
            this.app_dir = this.app_dir || this.node.get_pack().name;
            this.app_dir = this.app_dir.replace(/\\/g, '/').replace(/^\/+|\/+$/g, '');
            if (!this.node_src.is_exist(this.app_dir)) {
                console.log("directory not exist", this.app_dir);
                return this;
            }
            //开始复制目录到用户目录
            this.node_src.cp_dir_file(this.app_dir, this.user.get_path(this.app_name), [], true, false);
            return this;
        },
        create() {
            try {
                this.get_npm_dir();
                this.user_app = (0, exports.nodes)(this.user.get_path(this.app_name));
                //复制列表
                if (typeof copy_list == 'object' && Object.keys(copy_list).length > 0) {
                    if (copy_list.dir) {
                        const dir_list = Array.isArray(copy_list.dir) ? copy_list.dir : Object.values(copy_list.dir);
                        dir_list.forEach((val) => {
                            this.node.cp_dir_file(val, this.user_app.get_path(val), [], true, false);
                        });
                    }
                    if (copy_list.file) {
                        const file_list = Array.isArray(copy_list.file) ? copy_list.file : Object.values(copy_list.file);
                        file_list.forEach((val) => {
                            this.node.cp_file(val, this.user_app.get_path(val), true, false);
                        });
                    }
                }
                else {
                    this.create_src();
                }
                (typeof create == 'function' && create(this, 2));
                this.get_command();
            }
            catch (e) {
                console.log('error', e);
                console.log('argv', process.argv);
                console.log('import', import.meta);
                console.log('cwd', this.dirname);
                console.log('pathname', (0, exports.file_url)());
            }
            return this;
        }
    };
    return self.node.is_exist((npm || '.npmignore')) ? self.push() : self.create();
};
exports.npm_push_index = npm_push_index;
exports.default = exports.nodes;
